/*
 * module1.c
 * Basic demo program for demonstrating the use of the HERON-API to 
 * communicate between two processors. 
 *                                   
 *     PW    ver 2.0    30-3-00   changed for heron_api v2.3
 *     PW    ver 3.0    17-7-00 changed for CCS 1.2
 */

/* YOU need to set this value to reflect how your module is connected
to the next module. */
#define FIFONO	2	/* FIFO through which module communicates with next */

#include <stdio.h>
#include <string.h>
#include "heron1.h"
#include <std.h>


main()
{
}  

void maintask(void)
{
	HeronFIFO *rfifo;
	HeronFIFO *wfifo;
	int status;
	int data;
	int pattern[32];
	int results[32];

	int processors = 2;
	int i;
	int errors=0;
	/*
	 * All processors in the system are required to clear the config line
	 * to an in-active state. One all processors in a system performed a
	 * HeronConfigOff(), the config line goes in-active, and GDIO modules
	 * are only then able to send data onto C6xxx modules. 
	 * If you only use processor modules but no GDIO modules, then this
	 * doesn't matter to you and you can leave the config line in any
	 * state (active -> don't do a HeronConfigOff() or in-active -> do a
	 * HeronConfigOff() 
	 */
	HeronConfigOff();

	printf(" first HERON module has started running\n");

/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     exit(-1); 	       
     }
/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     exit(-1); 	
     } 

	while (processors)
	{
        /* Try to read the message from the next HERON module */
  	status = HeronRead(rfifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}

	printf("HERON module id %x has started replied\n", data);
	processors--;
	}

	errors = 0;


	for (i=0; i<32; i++)
	{
	pattern[i]=1<<i;
	results[i]=0;
	}

/* send the pattern to the next module */
  	status = HeronWrite(wfifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(wfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}
/* read the reply from the next HERON module */
  	status = HeronRead(rfifo,results,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}
/* check the data */
	for (i=0;i<32;i++)
	{
	if (results[i] != (1<<i)) errors++;
	}

	if (errors)
	{
		printf("%d Errors in bitwise echo test.\n", errors);
	}
	else
	{
		printf("Bitwise echo test completed successfully.\n");
	}

/* close the FIFOs */
	status = HeronClose(rfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing read fifo \n");
        exit(0); 
	}
	status = HeronClose(wfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing write fifo \n");
        exit(0); 
	}

   printf("finiished\n");

	/*
	 * 'exit()' will also exit from the server.
	 */
	exit( 0);
}
/*** End Of File ***/
