/*
 * heron2.c
 * Basic demo program for second DSP in a three DSP HERON-API example.
 *
 *     PW    ver 2.0    30-3-00   changed for heron_api v2.3
 *     PW    ver 3.0    17-1-00   changed for CCS 1.2
 */
                                                                    
#include <stdio.h>                                             
#include "heron1.h"
#include <std.h>

/* YOU need to set this value to reflect how your module is connected
to the first module. */
#define FIFONO	0	/* FIFO through which module communicates with the first module */

/* YOU need to set this value to reflect how your module is connected
to the third module. */
#define FIFO23       2/* FIFO through which module communicates with the third module */


main()
{
}  

void maintask(void)
{
	HeronFIFO *r1fifo;
	HeronFIFO *w1fifo;
	HeronFIFO *r3fifo;
	HeronFIFO *w3fifo;
	int status;
	int data;
	int pattern[32];
	
	/*
	 * All processors in the system are required to clear the config line
	 * to an in-active state. One all processors in a system performed a
	 * HeronConfigOff(), the config line goes in-active, and GDIO modules
	 * are only then able to send data onto C6xxx modules. 
	 * If you only use processor modules but no GDIO modules, then this
	 * doesn't matter to you and you can leave the config line in any
	 * state (active -> don't do a HeronConfigOff() or in-active -> do a
	 * HeronConfigOff() 
	 */
	HeronConfigOff();
             
             
	printf("Second HERON module has started running\n");

/* open the FIFO for reading from the first module*/
    r1fifo=HeronOpenFifo(FIFONO,"r");
    if (r1fifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     exit(-1); 	       
     }
/* open the FIFO for writing to the first module*/
    w1fifo=HeronOpenFifo(FIFONO,"w");
    if (w1fifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     exit(-1); 	
     } 
/* open the FIFO for reading from the third module*/
    r3fifo=HeronOpenFifo(FIFO23,"r");
    if (r3fifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFO23,heronerr);
     exit(-1); 	       
     }
/* open the FIFO for writing to the third module*/
    w3fifo=HeronOpenFifo(FIFO23,"w");
    if (w3fifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFO23,heronerr);
     exit(-1); 	
     } 

	
/* send a word to the first DSP */
        data = 0x2;
  	status = HeronWrite(w1fifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(w1fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}

/* read the ID from the third DSP */
  	status = HeronRead(r3fifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(r3fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFO23,heronerr);
	exit(-1);
	}
	
/* send the receievd ID to the first DSP */

  	status = HeronWrite(w1fifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(w1fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}


/* read a pattern from the first DSP */
  	status = HeronRead(r1fifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(r1fifo);
	}
	if (status != HERON_OK) 
	{
	printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}
	
/* send the receievd pattern to the third DSP */

  	status = HeronWrite(w3fifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(w3fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFO23,heronerr);
	exit(-1);
	}

/* read the pattern back from the third DSP */
  	status = HeronRead(r3fifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(r3fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot read fifo %d. Error %d\n",FIFO23,heronerr);
	exit(-1);
	}
	
/* send the receievd pattern to the first DSP */

  	status = HeronWrite(w1fifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(w1fifo);
	}
	if (status != HERON_OK)
	{
	printf("Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}

/* close the FIFOs */
	status = HeronClose(r1fifo);
	if (status != HERON_OK)
	{
	printf(" Error closing read fifo from module1 \n");
	exit(-1);
	}
	status = HeronClose(w1fifo);
	if (status != HERON_OK)
	{
		printf(" Error closing write fifo to module1 \n");
	exit(-1);
	}
	status = HeronClose(r3fifo);
	if (status != HERON_OK)
	{
		printf(" Error closing read fifo from module3\n");
	exit(-1);
	}
	status = HeronClose(w3fifo);
	if (status != HERON_OK)
	{
		printf(" Error closing write fifo to module3 \n");
	exit(-1);
	}


	while (1) {}            /* No real purpose, just for tracking */
}
/*** End Of File ***/
