/****************************************************************************/
/*																			*/
/*	EXAMPLE.C																*/
/*																			*/
/*	(c) 2000 Hunt Engineering												*/
/*	    All Rights Reserved													*/
/*																			*/
/*	This is an example C source file that shows how to use the Semaphore	*/
/*	Model functions of the HERON-API. In particular, this example shows		*/
/*	the use of the functions 'HeronSemOpenFifo' and 'HeronRead'.			*/
/*	The example program must be used with a second module that is providing */
/*	data to this program over a HERON FIFO. The second module can be an I/O */
/*	module, or another processing module.									*/
/*																			*/
/*	For a more detailed description of this example please refer to the		*/
/*	document 'sem_example.pdf' that accompanies this example source file.	*/
/*																			*/
/* 	Version	  Date	Version Author	Comments								*/
/*	-------	  ----	--------------	--------								*/
/*																			*/
/*    1.0	 25/9/00	  RW		Example first written					*/
/*																			*/
/****************************************************************************/
#include <std.h>
#include <tsk.h>
#include <sys.h> 
#include <stdio.h>


/* You must include the appropriate HERON-API header file for the HERON   */
/* module type you are using. The following line includes the header file */
/* for a HERON1. If you are using any other module type, you will need to */
/* change this line. */
#include "heron1.h"


/* You need to set this value to reflect which FIFO connection to use for */
/* the Semaphore based HERON read. */
#define FIFONO	2


#define N		100						  /* Number of buffers to process */


/* BUFSIZE is the number of 32-bit words in each buffer of data read.     */
#define BUFSIZE	256


unsigned int buffer0[BUFSIZE];
unsigned int buffer1[BUFSIZE];


/* A 'read_complete' SEM object must be created in the CDB file with	  */
/* which this source file is used. This SEM object is used by the HERON-  */
/* API to notify of a read completing. */
extern far SEM_Obj read_complete;    


/* A 'process' SEM object must be created in the CDB file with which	  */
/* this source file is used. This SEM object is used to control when the  */
/* 'processTask' task runs. */ 
extern far SEM_Obj process;    
                              
                              
HeronFIFO *handle;    
    

int main(void)
{
return(0);
}
   

/* A TSK object must be created in the CDB file for 'mainTask'. The		  */
/* priority of this task must be set to 2. */ 
void mainTask(void)
{
int buffer_count;
unsigned int *data;


printf("mainTask: The SEM example program is running\n"); 

/* Call HeronSemOpenFifo to open a FIFO for reading with the semaphore */
/* model. */
handle = HeronSemOpenFifo(FIFONO,"r",&read_complete);
if (handle == NULL) {
	printf("mainTask: Error opening fifo %d, error = %d\n",FIFONO,heronerr);
}

/* Release the Config line so that if an I/O module is connected, */
/* it can begin operation. */
HeronConfigOff();

/* Read N buffers */
for (buffer_count=0;buffer_count<N;buffer_count++) {
	if ((buffer_count & 0x01) == 0) data = buffer0;
	else data = buffer1;
	
	/* Start a read into buffer0 or buffer1 */
	HeronRead(handle,data,BUFSIZE);

	/* Wait for the completion of the read */
	SEM_pend(&read_complete,SYS_FOREVER);
	
	/* Post a semaphore to allow the processing task to run */
	SEM_post(&process);
}
printf("mainTask: Finished reading %d buffers\n",N);

return;
}


/* A TSK object must be created in the CDB file for 'processTask'. The	  */
/* priority of this task must be set to 1. */ 
void processTask(void)
{     
int i,j,sum;
unsigned int *data;


/* Process N buffers */
for (i=0;i<N;i++) {
	/* Wait for a buffer to be available */
	SEM_pend(&process,SYS_FOREVER);
	
	/* Process the buffer */
	if ((i & 0x01) == 0) data = buffer0;
	else data = buffer1;
	
	/* This example simply sums all of the elements of each buffer received */
	sum = 0;
	for (j=0;j<BUFSIZE;j++) {
		sum += data[j];
	}
	printf("processTask: Sum = %d\n",sum);
}

return;
}

