/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR USING HeronTestIo             */
/* FROM HERON-API IN A 'C6000 HERON SYSTEM.                 */
/* THE PROGRAM SHOWS THE USE OF THE FUNCTION TO KEEP AN     */
/* INPUT STREAM AND AN OUTPUT STREAM RUNNING CONTINUOUSLY   */
/* AND USES MULTIPLE DATA BUFFERS TO DECOUPLE VARIATIONS    */  
/* IN THE DATA RATES.                                       */
/*                                                          */
/*                                                          */
/*                                                          */
/*     PW    ver 1.0    3/4/00   demo first created         */
/*     PW    ver 3.0    17/7/00  changed for CCS 1.2        */
/*                                                          */  
/*                                                          */                                                                                                                         
/************************************************************/


/* YOU need to set this value to reflect how your input is connected
to this HERON module. */
#define INFIFONO    2           /* FIFO through which Data is received */
#define OUTFIFONO  0           /* FIFO through which Data is sent */
      
#include <std.h>    
#include <stdio.h>  
/* Use HERON-API for heron1 so you need the next include line*/
#include "heron1.h" 
             

/* defines used specifically in this program */
#define BUF_SIZE     1024   /* number of words per buffer */
#define TODO	10000          /* number of whole buffers to send */
#define NO_OF_BUFFS 100        /* number of circulating buffers */
      

main(void)
{  
} 


void maintask()
{
HeronFIFO *infifo;
HeronFIFO *outfifo;
int status;
int i;                                        
int *inbuffers[NO_OF_BUFFS];              
int nextinput=0;
int nextoutput=0;
int written=0;
int wrap=0;
  
printf("started\n");;  
                
for (i=0;i<NO_OF_BUFFS;i++)
{
inbuffers[i]=(int *)memalign(8,BUF_SIZE*sizeof(int));                
if (inbuffers[i]==NULL)
{ 
printf("error allocating buffer \n");
exit(-1);
} 
}                
                
                                             
/* open the input FIFO for reading*/
    infifo=HeronOpenFifo(INFIFONO,"r");
    if (infifo==NULL)
     {
      	printf("error opening input fifo for read\n"); 
	exit(-1);
     }

/* open the output FIFO for writing*/
    outfifo=HeronOpenFifo(OUTFIFONO,"w");
    if (outfifo==NULL)
     {
      	printf("error opening output fifo for write\n");
	exit(-1);
     }


/* RELEASE THE CONFIG LINE */
    HeronConfigOff(); 

/* start the first read, to prime the loop */
status = HeronRead(infifo,(int *)inbuffers[nextinput++],BUF_SIZE); 
if (status != HERON_IO_IN_PROGRESS)
		{
      	HeronDigioOut(0x3);
		exit(-1);
		}
                                                                       
while (written<TODO)
{            
 

status= HeronTestIo(infifo); /* check for completion of the input*/

if (status==HERON_OK)
{                     

	status = HeronRead(infifo,(int *)inbuffers[nextinput],BUF_SIZE);
	if (status != HERON_IO_IN_PROGRESS)
		{
      		printf("error reading \n");                                         
		exit(-1);
		}
	if (nextinput==0)
		{	                    
		wrap=1;	/* now inputting to buffer 0*/
		}                                                                                                                                                          
 	/*processing of data here, data is inbuffers[nextinput-1] */
        /*unless nextinput==0 when data is inbuffers[NO_OF_BUFFS-1] */
    
    if ((nextinput==nextoutput-1) || ((nextinput==(NO_OF_BUFFS-1))&(nextoutput==0)))
    /* we are now filling the buffer that is being sent */                         
        {       
        printf("buffers were overrun \n");                                         
	/*	exit(-1);*/
	}
    
        
	if (nextinput==NO_OF_BUFFS-1)
		{ 
	    	nextinput=0;
	    	}
	    else{         
        nextinput++;
        	}   
        
} 
 

status = HeronTestIo(outfifo); 	/* check for completion of the output*/ 
  			 	/* first time around there will be no outstanding */
  				/* I/O but this will return HERON_OK */
if (status==HERON_OK)
{                                  
   
	if ((nextoutput==(NO_OF_BUFFS-1))&&(wrap==1))
	{
		status = HeronWrite(outfifo,(int *)inbuffers[nextoutput],BUF_SIZE); 
		wrap=0;
		nextoutput=0;
		written++;
	}	  
	
	else if ((nextoutput<nextinput-1)||(wrap==1))
	{
	    status = HeronWrite(outfifo,(int *)inbuffers[nextoutput],BUF_SIZE);
	    nextoutput++;
	    written++;
	} 

}


}	
	
printf("finished \n");	    
while (1)IDL_run();
exit(0);


} /* end maintask */
