/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR USING THE HUNT ENGINEERING    */
/* HERON_API TO DOUBLE BUFFER DATA USING THE WAITIO FUNCTION*/
/* AS WOULD BE USED IN A REAL TIME                          */
/*                                                          */
/*                                                          */
/*                                                          */
/*     PW    ver 1.0    15-5-00    first written            */
/*     PW    ver 2.0    17-7-00    changed for CCS 1.2      */
/************************************************************/


/* YOU need to set this value to reflect how your I/O is connected
to your HERON module. */
#define FIFONO  2       /* FIFO through which I/O Data is received */

#include <stdio.h>
#include <std.h>

/* Use HERON-API for heron1 so you need the next include line*/
#include "heron1.h"


/* defines used specifically in this program*/
#define SAMPLE_SIZE     256   /* number of samples per chan         */
#define NO_OF_CHANNELS  1   /* use to set how many channels are used */
/* beware you must ensure that the correct data size for your I/O module is being used*/
#define BUF_SIZE SAMPLE_SIZE*NO_OF_CHANNELS     
#define NO_OF_BUFS 0x20000
    
HeronFIFO *infifo;
    
   
short   buf1[BUF_SIZE]; 
short   buf2[BUF_SIZE];                                
                       
               
main()
{
}        
        
       

maintask (void)
{

int status;                      
int channel0is;
int i;

          

printf("Hello from waitio demo.\n");

/* open the FIFO for reading*/
    infifo=HeronOpenFifo(FIFONO,"rd");
    if (infifo==NULL)
     {
       printf("error opening FIFO %d, error is %x\n",FIFONO,heronerr);
     }

    /* RELEASE THE CONFIG LINE */
    HeronConfigOff(); 
    
    /* start filling the frst buffer*/
    status = HeronRead(infifo,(int *)buf1,((BUF_SIZE)/2)); /* div 2 because 2 samples per int*/
   	if (status != HERON_IO_IN_PROGRESS)
	{       
    printf("error from read\n");
	exit(0);	
	}
	/* there is nothing we cna do until we receive the data so wait */         
    status = HeronWaitIo(infifo);    
	if (status != HERON_OK)
	{       
    printf("error from waitio\n");
	exit(0);	
	}                                                                 
	/* immediately re-start the read, but to a different buffer*/
	/* the HeronRestartRead function will be marginally quicker */
	/* than the HeronRead, but can only be used if the buffer   */
	/*length is the same as last time.*/ 
    HeronRestartRead(infifo,(int *)buf2);
    /* at this point we have a valid buffer in buf1, and we are gathering */
    /* new data into buf2 */
    
    /* HERE YOU SHOULD look at the end of the buffer #1 to find the position of ch0 */
    /* if you do not miss real time, then this position will remain for the rest of the */
    /* execution. */	        
    channel0is=0;                 
                         
    /* GRAB AND process DATA SAMPLES */
    for (i=0;i<NO_OF_BUFS;i+=2)    
    {        

	/* wait for buf2 to finish filling*/    
	status = HeronWaitIo(infifo);   
	if (status != HERON_OK)
	{       
    printf("error from waitio\n");
	exit(0);	
	}                                           
	/* restart the read into buf1*/
	Heron1RestartRead(infifo,(int *)buf1);	
	      
	/*sort the data from buf2*/
        /* do your processing of buffer 2 here */
                                      
	/* wait for buf1 to finish filling*/    
	status = HeronWaitIo(infifo);    
	if (status != HERON_OK)
	{       
    printf("error from waitio\n");
	exit(0);	
	}                                           
	/* restart the read into buf2*/
	Heron1RestartRead(infifo,(int *)buf2);	
	      
	/*sort the data from buf1*/
    /* do your processing of buffer 1 here */
                     	

    }

/* loops are done, so tidy up */
printf(" channel0 was %x \n",channel0is);
status = HeronClose(infifo);
	if (status != HERON_OK)
	{
	printf(" Error closing read fifo \n");
	exit(0);
	}


} /* end maintask */
